import { Vector } from '../Math/vector';
import { SolverStrategy } from './SolverStrategy';
/**
 * Possible broadphase collision pair identification strategies
 *
 * The default strategy is [[BroadphaseStrategy.DynamicAABBTree]] which uses a binary tree of axis-aligned bounding boxes to identify
 * potential collision pairs which is O(nlog(n)) faster.
 * @deprecated Unused in Excalibur, will be removed in v0.30
 */
export declare enum BroadphaseStrategy {
    DynamicAABBTree = 0
}
/**
 * Possible numerical integrators for position and velocity
 * @deprecated Unused in Excalibur, will be removed in v0.30
 */
export declare enum Integrator {
    Euler = 0
}
/**
 * The [[Physics]] object is the global configuration object for all Excalibur physics.
 * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
 */
export declare class Physics {
    /**
     * Global acceleration that is applied to all vanilla actors that have a [[CollisionType.Active|active]] collision type.
     * Global acceleration won't effect [[Label|labels]], [[ScreenElement|ui actors]], or [[Trigger|triggers]] in Excalibur.
     *
     * This is a great way to globally simulate effects like gravity.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static acc: Vector;
    /**
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static get gravity(): Vector;
    static set gravity(v: Vector);
    /**
     * Globally switches all Excalibur physics behavior on or off.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static enabled: boolean;
    /**
     * Gets or sets the broadphase pair identification strategy.
     *
     * The default strategy is [[BroadphaseStrategy.DynamicAABBTree]] which uses a binary tree of axis-aligned bounding boxes to identify
     * potential collision pairs which is O(nlog(n)) faster.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static broadphaseStrategy: BroadphaseStrategy;
    /**
     * Gets or sets the global collision resolution strategy (narrowphase).
     *
     * The default is [[SolverStrategy.Arcade]] which performs simple axis aligned arcade style physics.
     *
     * More advanced rigid body physics are enabled by setting [[SolverStrategy.Realistic]] which allows for complicated
     * simulated physical interactions.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static collisionResolutionStrategy: SolverStrategy;
    /**
     * The default mass to use if none is specified
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static defaultMass: number;
    /**
     * Gets or sets the position and velocity positional integrator, currently only Euler is supported.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static integrator: Integrator;
    /**
     * Configures Excalibur to use "arcade" physics. Arcade physics which performs simple axis aligned arcade style physics.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static useArcadePhysics(): void;
    /**
     * Configures Excalibur to use rigid body physics. Rigid body physics allows for complicated
     * simulated physical interactions.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static useRealisticPhysics(): void;
    /**
     * Factor to add to the RigidBody BoundingBox, bounding box (dimensions += vel * dynamicTreeVelocityMultiplier);
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static dynamicTreeVelocityMultiplier: number;
    /**
     * Pad RigidBody BoundingBox by a constant amount
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static boundsPadding: number;
    /**
     * Number of position iterations (overlap) to run in the solver
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static positionIterations: number;
    /**
     * Number of velocity iteration (response) to run in the solver
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static velocityIterations: number;
    /**
     * Amount of overlap to tolerate in pixels
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static slop: number;
    /**
     * Amount of positional overlap correction to apply each position iteration of the solver
     * O - meaning no correction, 1 - meaning correct all overlap
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static steeringFactor: number;
    /**
     * Warm start set to true re-uses impulses from previous frames back in the solver
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static warmStart: boolean;
    /**
     * By default bodies do not sleep
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static bodiesCanSleepByDefault: boolean;
    /**
     * Surface epsilon is used to help deal with surface penetration
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static surfaceEpsilon: number;
    /**
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static sleepEpsilon: number;
    /**
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static wakeThreshold: number;
    /**
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static sleepBias: number;
    /**
     * Enable fast moving body checking, this enables checking for collision pairs via raycast for fast moving objects to prevent
     * bodies from tunneling through one another.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static checkForFastBodies: boolean;
    /**
     * Disable minimum fast moving body raycast, by default if ex.Physics.checkForFastBodies = true Excalibur will only check if the
     * body is moving at least half of its minimum dimension in an update. If ex.Physics.disableMinimumSpeedForFastBody is set to true,
     * Excalibur will always perform the fast body raycast regardless of speed.
     * @deprecated Use engine args to configure Physics `new ex.Engine({physics: {...}})`, will be removed in v0.30
     */
    static disableMinimumSpeedForFastBody: boolean;
}
